
#pragma once

///////////////////////////////////////////////////////////////////////

#define HID_DEVICE_SUCCESS			0x00
#define HID_DEVICE_NOT_FOUND		0x01
#define HID_DEVICE_NOT_OPENED		0x02
#define HID_DEVICE_ALREADY_OPENED	0x03
#define HID_DEVICE_TRANSFER_TIMEOUT	0x04
#define HID_DEVICE_TRANSFER_FAILED	0x05
#define HID_DEVICE_HANDLE_ERROR		0x06
#define HID_DEVICE_UNKNOWN_ERROR	0xFF

///////////////////////////////////////////////////////////////////////
class HID_Controller
{
public:
	HID_Controller();
	virtual ~HID_Controller();
public:

	bool IsEnable();
	bool IsOpen();

	void Init();

	bool Open( int nVendorID, int nProductID );
	void Close();

public:
	//
	BYTE WriteFile( BYTE* pBuffer, DWORD dwBufferSize );
	//
	BYTE ReadFile( BYTE* pBuffer, DWORD dwBufferSize );
	//
	BYTE FlushBuffer();

	BYTE GetFeature( BYTE* pBuffer );
	BYTE SetFeature( BYTE* pBuffer );

private:
	HANDLE _Open( int nVendorID, int nProductID );
protected:
	HANDLE	 m_handle;		//HIDHandle
	bool	 m_bOpen;
	UINT	 m_uGetReportTimeOut;
	UINT	 m_uSetReportTimeOut;

	OVERLAPPED	m_Read;
	OVERLAPPED	m_Write;

	WORD	m_wInReportBufferLength;
	WORD	m_wOutReportBufferLength;
	WORD	m_wFeatReportBufferLength;

	WORD	m_wInBufferUsed;

	WORD	m_VendorId;		//	x_[ID
	WORD	m_ProductId;	//@v_NgID
};


///////////////////////////////////////////////////////////////////////
#define ONAHO_STATUS_MAX	16	//Iizp̓ǂݏoCgTCY
class COnahole: public HID_Controller
{
public:
	COnahole();
	virtual ~COnahole();
public:

	bool ChangeStatus( BYTE* pData, int nDataSize );

	bool IsPause();
	void Pause();
	void Start();

	void Quit();

	bool SetPattern( int nPattern );
	bool SetLevel( int nLevel );
	bool SetSpeed( int nLevel );

private:
	BYTE m_OnahoStatus[ONAHO_STATUS_MAX];	//16oCgf[^
	
	static const int PlayIndex = 6;		//PlayPause
	static const int PatternIndex = 7;	//p^[
	static const int LevelIndex	= 8;	//x

public:
	bool	Set( int pattern=-1, int speed=-1 );	//ނƑx

	bool	AddPattern();
	bool	SubPattern();
	bool	AddSpeed();
	bool	SubSpeed();
	bool	SendCommand();
	bool	CatchReaction( char *pBuffer=NULL );

	int		GetPattern(){ return this->m_nPattern; }
	int		GetSpeed()	{ return this->m_nSpeed; }

	void	AskStatus();
	int		GetBattery();
	int		IsAC();


	BYTE	m_Buffer[16];

	int		m_nPattern;
	int		m_nSpeed;
	bool	m_bPause;

};